//
//  ViewController.m
//  TestURLCache
//
//  Created by Ben Blaukopf on 10/10/2014.
//  Copyright (c) 2014 airsource. All rights reserved.
//

#import "ViewController.h"
#import "AppDelegate.h"
unsigned long shmWalSize;

@interface ViewController () <NSURLConnectionDataDelegate>

@end

@implementation ViewController
{
    int fetches;
    unsigned long dataSize;
}

@synthesize fetchButton, cacheableFetchButton, label, errorLabel;
- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    
    NSLog(@"Initial usage: %lu", (unsigned long)[[NSURLCache sharedURLCache] currentDiskUsage]);
    errorLabel.text = @"";
    [self inspectCache:nil];

}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(IBAction)fetch:(id)sender
{
    fetchButton.enabled = NO;
    cacheableFetchButton.enabled = NO;
    fetches = 10;
    [self doFetch];
}

-(void)doFetch
{
    static NSUInteger i;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        NSArray *a = [ViewController cacheStats][1];
        i = a.count;
    });
    NSString *fetch = [NSString stringWithFormat:DOWNLOAD_URL, (unsigned long)i];
    i++;
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:fetch] cachePolicy:NSURLRequestReturnCacheDataElseLoad timeoutInterval:10];
    dataSize = 0;
    
    NSURLConnection *conn = [[NSURLConnection alloc] initWithRequest:request delegate:self];
    [conn start];
}

-(IBAction)cacheableFetch:(id)sender
{
    fetches = 1;
    fetchButton.enabled = NO;
    cacheableFetchButton.enabled = NO;
    NSString *fetch = [NSString stringWithFormat:DOWNLOAD_URL, (unsigned long)0];
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:fetch] cachePolicy:NSURLRequestReturnCacheDataElseLoad timeoutInterval:10];
    NSURLConnection *conn = [[NSURLConnection alloc] initWithRequest:request delegate:self];
    [conn start];
}

#pragma mark NSURLConnection Delegate Methods

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
    // A response has been received, this is where we initialize the instance var you created
    // so that we can append data to it in the didReceiveData method
    // Furthermore, this method is called each time there is a redirect so reinitializing it
    // also serves to clear it
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    // Append the new data to the instance variable you declared
    dataSize += data.length;
}

-(IBAction)inspectCache:(id)sender
{
    NSArray* stats = [ViewController cacheStats];
    NSURLCache *cache = [NSURLCache sharedURLCache];
    NSNumber *n = stats[0];
    label.text = [NSString stringWithFormat:@"Disk: %luKB (vs %luKB)\ninc SHM/WAL %luKB\nMemory: %luKB\nMax: %luKB",
                  (unsigned long)(n.unsignedLongValue / 1024),
                  (unsigned long)(cache.currentDiskUsage/1024),
                  (unsigned long)((n.unsignedLongValue+shmWalSize) / 1024),
                  (unsigned long)(cache.currentMemoryUsage/1024),
                  (unsigned long)(cache.diskCapacity/1024)];
}

+(NSArray*)statsForDir:(NSString*)dir
{
    NSFileManager *fm = [NSFileManager defaultManager];
    
    // Accumulate the size for each file in fsCachedData, and the main dir.
    NSArray *filelist= [fm contentsOfDirectoryAtPath:dir error:nil];
    NSMutableArray *results = [[NSMutableArray alloc] initWithCapacity:2];
    size_t sz = 0;
    NSMutableArray *allFiles = [[NSMutableArray alloc] initWithArray:filelist];
    for(NSString * f in filelist)
    {
        NSString *path = [dir stringByAppendingFormat:@"/%@", f];
        
        
        NSError *attributesError = nil;
        NSDictionary *fileAttributes = [[NSFileManager defaultManager] attributesOfItemAtPath:path error:&attributesError];
        if([fileAttributes.fileType isEqualToString:NSFileTypeDirectory])
        {
            NSArray *subresults = [self statsForDir:path];
            NSNumber *n = subresults[0];
            sz += n.unsignedLongValue;
            [allFiles addObjectsFromArray:subresults[1]];
        }
        else
        {
            // We ignore the WAL and SHM files, as the operating system (at least on iOS 7) does too.
            if([f isEqualToString:@"Cache.db-shm"] || [f isEqualToString:@"Cache.db-wal"])
            {
                shmWalSize += fileAttributes.fileSize;
                continue;
            }
            sz += fileAttributes.fileSize;
        }
    }
    results[0] = @(sz);
    results[1] = allFiles;
    return results;
}

+(NSArray*)cacheStats
{
    shmWalSize = 0;

    NSArray *docPaths = NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES);
    NSString *dir = (NSString*)[docPaths objectAtIndex:0];
    dir = [dir stringByAppendingString:@"/uk.co.airsource.TestURLCache/nsurlcache"];
    
    NSArray * stats = [self statsForDir:dir];
    return stats;
}

-(IBAction)clearCache:(id)sender
{
    [[NSURLCache sharedURLCache] removeAllCachedResponses];
    NSLog(@"Cleared cache");
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1.0 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self inspectCache:nil];
    });
}


- (NSCachedURLResponse *)connection:(NSURLConnection *)connection
                  willCacheResponse:(NSCachedURLResponse*)cachedResponse {
    // Return nil to indicate not necessary to store a cached response for this connection
    // Entries in cache:
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1.0 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self inspectCache:nil];
    });

    
    return cachedResponse;
}


- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
    // The request is complete and data has been received
    // You can parse the stuff in your instance variable now
    fetches--;
    errorLabel.text = @"Fetch succeeded";
    if(fetches == 0)
    {
        fetchButton.enabled = YES;
        cacheableFetchButton.enabled = YES;

        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1.0 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self inspectCache:nil];
        });
    }
    else
    {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self doFetch];
        });
    }

    
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
    // The request has failed for some reason!
    // Check the error var
    fetchButton.enabled = YES;
    errorLabel.text = @"Fetch failed";
    fetches = 0;
}
@end
